@@ -62,7 +62,7 @@ namespace BeagleWksSSParserInternal
 //! Internal: the cell of a BeagleWksSSParser
 struct Cell : public MWAWCell {
   //! constructor
-  Cell(Vec2i pos=Vec2i(0,0)) : MWAWCell(), m_content(), m_formula(-1), m_isEmpty(false)
+  explicit Cell(MWAWVec2i pos=MWAWVec2i(0,0)) : MWAWCell(), m_content(), m_formula(-1), m_isEmpty(false)
   {
     setPosition(pos);
   }
@@ -81,7 +81,31 @@ struct Spreadsheet {
   {
   }
   //! try to associate a formula to a cell
-  bool addFormula(Vec2i const &cellPos, std::vector<MWAWCellContent::FormulaInstruction> const &formula);
+  bool addFormula(MWAWVec2i const &cellPos, std::vector<MWAWCellContent::FormulaInstruction> const &formula);
+  //! returns the height of a row in point
+  double getRowHeight(int row) const
+  {
+    if (row<0 || row>=(int) m_heightRows.size() || m_heightRows[size_t(row)]<0)
+      return 16.;
+    return m_heightRows[size_t(row)];
+  }
+  //! returns the height of a row in point and updated repeated row
+  double getRowHeight(int row, int &numRepeated) const
+  {
+    double res=getRowHeight(row);
+    numRepeated=1;
+    if (row<0 || row>=(int) m_heightRows.size())
+      numRepeated=1000;
+    else {
+      for (size_t r=size_t(row+1); r<m_heightRows.size(); ++r) {
+        double nextH=getRowHeight(row+1);
+        if (nextH<res || nextH>res)
+          break;
+        ++numRepeated;
+      }
+    }
+    return res;
+  }
   //! convert the m_widthCols, m_heightRows in a vector of of point size
   static std::vector<float> convertInPoint(std::vector<int> const &list,
       float defSize)
@@ -117,7 +141,7 @@ struct Spreadsheet {
   int m_lastReadRow;
 };
 
-bool Spreadsheet::addFormula(Vec2i const &cellPos, std::vector<MWAWCellContent::FormulaInstruction> const &formula)
+bool Spreadsheet::addFormula(MWAWVec2i const &cellPos, std::vector<MWAWCellContent::FormulaInstruction> const &formula)
 {
   for (size_t c=0; c < m_cells.size(); ++c) {
     if (m_cells[c].position()!=cellPos)
@@ -307,10 +331,10 @@ libmwaw::DebugFile &BeagleWksSSParser::rsrcAscii()
 ////////////////////////////////////////////////////////////
 // position and height
 ////////////////////////////////////////////////////////////
-Vec2f BeagleWksSSParser::getPageLeftTop() const
+MWAWVec2f BeagleWksSSParser::getPageLeftTop() const
 {
-  return Vec2f(float(getPageSpan().getMarginLeft()),
-               float(getPageSpan().getMarginTop()+m_state->m_headerHeight/72.0));
+  return MWAWVec2f(float(getPageSpan().getMarginLeft()),
+                   float(getPageSpan().getMarginTop()+m_state->m_headerHeight/72.0));
 }
 
 ////////////////////////////////////////////////////////////
@@ -334,8 +358,7 @@ void BeagleWksSSParser::newPage(int number)
 ////////////////////////////////////////////////////////////
 void BeagleWksSSParser::parse(librevenge::RVNGSpreadsheetInterface *docInterface)
 {
-  assert(getInput().get() != 0);
-  if (!checkHeader(0L))  throw(libmwaw::ParseException());
+  if (!getInput().get() || !checkHeader(0L))  throw(libmwaw::ParseException());
   bool ok = false;
   try {
     // create the asciiFile
@@ -547,20 +570,20 @@ bool BeagleWksSSParser::readPrintInfo()
   if (!info.read(input)) return false;
   f << "Entries(PrintInfo):"<< info;
 
-  Vec2i paperSize = info.paper().size();
-  Vec2i pageSize = info.page().size();
+  MWAWVec2i paperSize = info.paper().size();
+  MWAWVec2i pageSize = info.page().size();
   if (pageSize.x() <= 0 || pageSize.y() <= 0 ||
       paperSize.x() <= 0 || paperSize.y() <= 0) return false;
 
   // define margin from print info
-  Vec2i lTopMargin= -1 * info.paper().pos(0);
-  Vec2i rBotMargin=info.paper().pos(1) - info.page().pos(1);
+  MWAWVec2i lTopMargin= -1 * info.paper().pos(0);
+  MWAWVec2i rBotMargin=info.paper().pos(1) - info.page().pos(1);
 
   // move margin left | top
   int decalX = lTopMargin.x() > 14 ? lTopMargin.x()-14 : 0;
   int decalY = lTopMargin.y() > 14 ? lTopMargin.y()-14 : 0;
-  lTopMargin -= Vec2i(decalX, decalY);
-  rBotMargin += Vec2i(decalX, decalY);
+  lTopMargin -= MWAWVec2i(decalX, decalY);
+  rBotMargin += MWAWVec2i(decalX, decalY);
 
   // decrease right | bottom
   int rightMarg = rBotMargin.x() -10;
@@ -641,7 +664,7 @@ bool BeagleWksSSParser::readChart()
   if (val!=0x12) f << "f1=" << val << ",";
   bool findRange=false;
   shared_ptr<BeagleWksSSParserInternal::Chart> chart(new BeagleWksSSParserInternal::Chart(m_state->m_spreadsheetName, *this));
-  Box2i serieRange;
+  MWAWBox2i serieRange;
   for (int i=0; i<2; ++i) {
     long actPos=input->tell();
     int dSz=(int) input->readULong(2);
@@ -676,9 +699,9 @@ bool BeagleWksSSParser::readChart()
           row=row*10+int(name[cPos++]-'0');
         }
         if (c==0)
-          serieRange.setMin(Vec2i(col-1,row-1));
+          serieRange.setMin(MWAWVec2i(col-1,row-1));
         else {
-          serieRange.setMax(Vec2i(col-1,row-1));
+          serieRange.setMax(MWAWVec2i(col-1,row-1));
           findRange=true;
           break;
         }
@@ -858,8 +881,8 @@ bool BeagleWksSSParser::readChart()
     chart->setDataType(serieType, true);
     for (int r=serieRange[0][1]; r<serieRange[1][1]; ++r) {
       MWAWChart::Series series;
-      series.m_range=Box2i(Vec2i(serieRange[0][0], r),
-                           Vec2i(serieRange[1][0], r));
+      series.m_range=MWAWBox2i(MWAWVec2i(serieRange[0][0], r),
+                               MWAWVec2i(serieRange[1][0], r));
       series.m_type=serieType;
       unsigned char gray=(unsigned char)((r%4)*30);
       series.m_style.m_lineWidth=1;
@@ -954,7 +977,7 @@ bool BeagleWksSSParser::readRowSheet(BeagleWksSSParserInternal::Spreadsheet &she
   for (int i=0; i < N; ++i) {
     pos=input->tell();
     if (pos==endPos) break;
-    BeagleWksSSParserInternal::Cell cell(Vec2i(i, row));
+    BeagleWksSSParserInternal::Cell cell(MWAWVec2i(i, row));
     if (!readCellSheet(cell))
       return false;
     sheet.m_cells.push_back(cell);
@@ -1126,15 +1149,15 @@ bool BeagleWksSSParser::readCellSheet(BeagleWksSSParserInternal::Cell &cell)
       }
       switch ((wh>>5)&3) {
       case 0:
-        cell.setHAlignement(MWAWCell::HALIGN_LEFT);
+        cell.setHAlignment(MWAWCell::HALIGN_LEFT);
         f << "left,";
         break;
       case 1:
-        cell.setHAlignement(MWAWCell::HALIGN_RIGHT);
+        cell.setHAlignment(MWAWCell::HALIGN_RIGHT);
         f << "right,";
         break;
       case 2:
-        cell.setHAlignement(MWAWCell::HALIGN_CENTER);
+        cell.setHAlignment(MWAWCell::HALIGN_CENTER);
         f << "center,";
         break;
       case 3: // default
@@ -1195,7 +1218,7 @@ bool BeagleWksSSParser::readCellSheet(BeagleWksSSParserInternal::Cell &cell)
     else if (type!=4) {
       // change the reference date from 1/1/1904 to 1/1/1900
       if (format.m_format==MWAWCell::F_DATE)
-        content.setValue(value+1462.);
+        content.setValue(value+1460.);
       else if (format.m_format==MWAWCell::F_TIME)
         content.setValue(std::fmod(value,1));
       else
@@ -1345,8 +1368,8 @@ bool BeagleWksSSParser::readFormula(BeagleWksSSParserInternal::Spreadsheet &shee
     }
     std::vector<MWAWCellContent::FormulaInstruction> formula;
     std::string error("");
-    if (m_structureManager->readFormula(pos+6+dataSz, Vec2i(col,row), formula, error))
-      sheet.addFormula(Vec2i(col,row), formula);
+    if (m_structureManager->readFormula(pos+6+dataSz, MWAWVec2i(col,row), formula, error))
+      sheet.addFormula(MWAWVec2i(col,row), formula);
     else
       f << "###";
     for (size_t l=0; l < formula.size(); ++l)
@@ -1374,7 +1397,7 @@ bool BeagleWksSSParser::sendPageFrames()
 
 bool BeagleWksSSParser::sendFrame(BeagleWksStructManager::Frame const &frame)
 {
-  MWAWPosition fPos(Vec2f(0,0), frame.m_dim, librevenge::RVNG_POINT);
+  MWAWPosition fPos(MWAWVec2f(0,0), frame.m_dim, librevenge::RVNG_POINT);
 
   fPos.setPagePos(frame.m_page > 0 ? frame.m_page : 1, frame.m_origin);
   fPos.setRelativePosition(MWAWPosition::Page);
@@ -1408,7 +1431,7 @@ bool BeagleWksSSParser::sendPicture
   if (!m_structureManager->readPicture(pId, data))
     return false;
 
-  listener->insertPicture(pictPos, data, "image/pict", style);
+  listener->insertPicture(pictPos, MWAWEmbeddedObject(data, "image/pict"), style);
   return true;
 }
 
@@ -1461,34 +1484,36 @@ bool BeagleWksSSParser::sendSpreadsheet()
   size_t numCell = sheet.m_cells.size();
 
   int prevRow = -1;
-  listener->openSheet(sheet.convertInPoint(sheet.m_widthCols,76), librevenge::RVNG_POINT, m_state->m_spreadsheetName);
+  listener->openSheet(sheet.convertInPoint(sheet.m_widthCols,76), librevenge::RVNG_POINT, std::vector<int>(), m_state->m_spreadsheetName);
   sendPageFrames();
   // send the chart
   for (size_t i = 0; i < m_state->m_chartList.size(); ++i) {
     if (!m_state->m_chartList[i]) continue;
     BeagleWksSSParserInternal::Chart &chart=*(m_state->m_chartList[i]);
     // chart have no position, so we create one
-    chart.setDimension(Vec2i(200,200));
-    MWAWPosition fPos(Vec2f(0,0), Vec2f(200,200), librevenge::RVNG_POINT);
-    fPos.setPagePos(1, Vec2f(float(i)*200,200));
+    chart.setDimension(MWAWVec2i(200,200));
+    MWAWPosition fPos(MWAWVec2f(0,0), MWAWVec2f(200,200), librevenge::RVNG_POINT);
+    fPos.setPagePos(1, MWAWVec2f(float(i)*200,200));
     fPos.setRelativePosition(MWAWPosition::Page);
     listener->insertChart(fPos, chart);
   }
-  std::vector<float> rowHeight = sheet.convertInPoint(sheet.m_heightRows,16);
   for (size_t i = 0; i < numCell; i++) {
     BeagleWksSSParserInternal::Cell const &cell= sheet.m_cells[i];
-    // FIXME: openSheetRow/openSheetCell must do that
-    if (cell.position()[1] != prevRow) {
-      while (cell.position()[1] > prevRow) {
-        if (prevRow != -1)
-          listener->closeSheetRow();
-        prevRow++;
-        if (prevRow < int(rowHeight.size()))
-          listener->openSheetRow(rowHeight[size_t(prevRow)], librevenge::RVNG_POINT);
-        else
-          listener->openSheetRow(16, librevenge::RVNG_POINT);
+    if (cell.position()[1]>prevRow+1) {
+      while (cell.position()[1] > prevRow+1) {
+        if (prevRow != -1) listener->closeSheetRow();
+        int numRepeat;
+        double h=sheet.getRowHeight(prevRow+1, numRepeat);
+        if (cell.position()[1]<prevRow+1+numRepeat)
+          numRepeat=cell.position()[1]-1-prevRow;
+        listener->openSheetRow((float) h, librevenge::RVNG_POINT, numRepeat);
+        prevRow+=numRepeat;
       }
     }
+    if (cell.position()[1] > prevRow) {
+      if (prevRow != -1) listener->closeSheetRow();
+      listener->openSheetRow((float) sheet.getRowHeight(++prevRow), librevenge::RVNG_POINT);
+    }
     listener->openSheetCell(cell, cell.m_content);
     if (cell.m_content.m_textEntry.valid()) {
       listener->setFont(cell.isFontSet() ? cell.getFont() : MWAWFont());
