@@ -569,7 +569,7 @@ bool BeagleWksStructManager::readFontStyle(MWAWEntry const &entry)
   return true;
 }
 
-bool BeagleWksStructManager::readPicture(int pId, librevenge::RVNGBinaryData &pict)
+bool BeagleWksStructManager::readPicture(int pId, librevenge::RVNGBinaryData &pict, bool readEDTP)
 {
   MWAWRSRCParserPtr rsrcParser = m_parserState->m_rsrcParser;
   if (!rsrcParser) {
@@ -583,11 +583,11 @@ bool BeagleWksStructManager::readPicture(int pId, librevenge::RVNGBinaryData &pi
 
   std::multimap<std::string, MWAWEntry> &entryMap =
     rsrcParser->getEntriesMap();
-  std::multimap<std::string, MWAWEntry>::const_iterator it
-    =entryMap.find("edtp");
+  std::string const what(readEDTP ? "edtp" : "PICT");
+  std::multimap<std::string, MWAWEntry>::const_iterator it=entryMap.find(what);
   MWAWEntry pictEntry;
   while (it!=entryMap.end()) {
-    if (it->first!="edtp")
+    if (it->first!=what)
       break;
     MWAWEntry const &entry=it++->second;
     if (entry.id()!=pId)
