@@ -112,7 +112,7 @@ struct Block {
     long m_dataSize;
   };
   //! constructor
-  Block(BlockHeader const &header) : m_header(header), m_zoneList()
+  explicit Block(BlockHeader const &header) : m_header(header), m_zoneList()
   {
   }
   //! constructor given a zone
@@ -158,15 +158,17 @@ class Cell : public MWAWCell
 {
 public:
   /// constructor
-  Cell() : m_content() { }
+  Cell() : m_content(), m_pictEntry() { }
   //! returns true if the cell do contain any content
   bool isEmpty() const
   {
-    return m_content.empty() && !hasBorders();
+    return m_content.empty() && !hasBorders() && !m_pictEntry.valid();
   }
 
   //! the cell content
   MWAWCellContent m_content;
+  //! picture
+  MWAWEntry m_pictEntry;
 };
 
 /** a field of a GreatWksDBParser */
@@ -175,7 +177,8 @@ struct Field {
   enum Type { F_Unknown, F_Text, F_Number, F_Date, F_Time, F_Memo, F_Picture, F_Formula, F_Summary };
   //! constructor
   Field() : m_type(F_Unknown), m_id(-1), m_name(""), m_format(), m_linkZone(0), m_recordBlock(),
-    m_formula(), m_summaryType(0), m_summaryField(0), m_isSequence(false), m_firstNumber(1), m_incrementNumber(1), m_extra("")
+    m_formula(), m_summaryType(0), m_summaryField(0), m_isSequence(false), m_firstNumber(1), m_incrementNumber(1),
+    m_extra("")
   {
   }
   //! update the cell to correspond to the final data
@@ -213,6 +216,7 @@ struct Field {
   int m_firstNumber;
   //! the increment number (in case of progression sequence)
   int m_incrementNumber;
+
   //! extra data
   std::string m_extra;
 };
@@ -244,7 +248,7 @@ bool Field::updateCell(int row, int numRow, Cell &cell) const
     instr.m_position[0][0]=instr.m_position[1][0]=m_summaryField;
     instr.m_position[0][1]=0;
     instr.m_position[1][1]=numRow-1;
-    instr.m_positionRelative[0]=instr.m_positionRelative[1]=Vec2b(false,false);
+    instr.m_positionRelative[0]=instr.m_positionRelative[1]=MWAWVec2b(false,false);
     formula.push_back(instr);
     instr.m_type=MWAWCellContent::FormulaInstruction::F_Operator;
     instr.m_content=")";
@@ -253,7 +257,7 @@ bool Field::updateCell(int row, int numRow, Cell &cell) const
   }
   // change the reference date from 1/1/1904 to 1/1/1900
   if (m_type==F_Date && cell.m_content.isValueSet())
-    cell.m_content.setValue(cell.m_content.m_value+1462.);
+    cell.m_content.setValue(cell.m_content.m_value+1460.);
   // and try to update the 1D formula in 2D
   for (size_t i=0; i<formula.size(); ++i) {
     MWAWCellContent::FormulaInstruction &instr=formula[i];
@@ -316,7 +320,7 @@ public:
   {
   }
   //! add a cell data in one given position
-  bool addCell(Vec2i const &pos, Cell const &cell)
+  bool addCell(MWAWVec2i const &pos, Cell const &cell)
   {
     if (pos[0]<0 || pos[0]>=int(m_fieldList.size()) || pos[1]<0) {
       MWAW_DEBUG_MSG(("GreatWksDBParserInternal::Database::addCell: the cell position seems bad\n"));
@@ -430,10 +434,13 @@ void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType type
     return;
   }
 
-  assert(m_parser);
-
+  GreatWksDBParser *parser=dynamic_cast<GreatWksDBParser *>(m_parser);
+  if (!parser) {
+    MWAW_DEBUG_MSG(("GreatWksDBParserInternal::SubDocument::parse: no parser\n"));
+    return;
+  }
   long pos = m_input->tell();
-  static_cast<GreatWksDBParser *>(m_parser)->sendHF(m_id);
+  parser->sendHF(m_id);
   m_input->seek(pos, librevenge::RVNG_SEEK_SET);
 }
 
@@ -486,8 +493,7 @@ bool GreatWksDBParser::sendHF(int id)
 ////////////////////////////////////////////////////////////
 void GreatWksDBParser::parse(librevenge::RVNGSpreadsheetInterface *docInterface)
 {
-  assert(getInput().get() != 0);
-  if (!checkHeader(0L))  throw(libmwaw::ParseException());
+  if (!getInput().get() || !checkHeader(0L))  throw(libmwaw::ParseException());
   bool ok = false;
   try {
     // create the asciiFile
@@ -805,7 +811,7 @@ bool GreatWksDBParser::readHeader()
       MWAWEntry entry;
       entry.setBegin(ptr);
       zones.push_back(entry);
-      if (!ptr) continue;
+      if (!ptr || ptr==0xFFFF) continue;
       f << "Zone" << 9+i << "A=" << std::hex << ptr << std::dec << ",";
     }
   }
@@ -818,7 +824,7 @@ bool GreatWksDBParser::readHeader()
 
   for (size_t i=0; i<zones.size(); ++i) {
     MWAWEntry &entry=zones[i];
-    if (!entry.begin()) continue;
+    if (!entry.begin() || entry.begin()==0xFFFF) continue;
     if (!checkSmallZone(entry)) {
       MWAW_DEBUG_MSG(("GreatWksDBParser::readHeader: find a bad zone %d\n", (int) i));
       entry.setLength(0);
@@ -1226,7 +1232,7 @@ bool GreatWksDBParser::readRowRecords(MWAWEntry const &entry)
       cell.m_content.m_contentType=MWAWCellContent::C_TEXT;
       cell.m_content.m_textEntry.setBegin(input->tell());
       cell.m_content.m_textEntry.setLength(sSz);
-      database.addCell(Vec2i(field.m_id, entry.id()), cell);
+      database.addCell(MWAWVec2i(field.m_id, entry.id()), cell);
 
       std::string text("");
       for (int i=0; i<sSz; ++i) text+=(char) input->readULong(1);
@@ -1245,7 +1251,7 @@ bool GreatWksDBParser::readRowRecords(MWAWEntry const &entry)
       }
       // formula will be will later
       cell.m_content.m_contentType=MWAWCellContent::C_FORMULA;
-      database.addCell(Vec2i(field.m_id, entry.id()), cell);
+      database.addCell(MWAWVec2i(field.m_id, entry.id()), cell);
       f << "summary,";
       input->seek(pos+10, librevenge::RVNG_SEEK_SET);
       break;
@@ -1262,6 +1268,9 @@ bool GreatWksDBParser::readRowRecords(MWAWEntry const &entry)
       }
       double value;
       bool isNan;
+      /*fixme: this does not work for time's field as value is a very
+        big number and the double precision is too small, ie. we do not
+        retrieve the decimal part -> time is always 0:0:0 */
       if (!input->readDouble10(value, isNan)) {
         static bool first=true;
         if (first) {
@@ -1274,7 +1283,7 @@ bool GreatWksDBParser::readRowRecords(MWAWEntry const &entry)
       else {
         cell.m_content.m_contentType=MWAWCellContent::C_NUMBER;
         cell.m_content.setValue(value);
-        database.addCell(Vec2i(field.m_id, entry.id()), cell);
+        database.addCell(MWAWVec2i(field.m_id, entry.id()), cell);
         f << value;
       }
       input->seek(pos+10, librevenge::RVNG_SEEK_SET);
@@ -1289,6 +1298,9 @@ bool GreatWksDBParser::readRowRecords(MWAWEntry const &entry)
         ascii().addNote(f.str().c_str());
       }
       if (pSz) {
+        cell.m_pictEntry.setBegin(pos+4);
+        cell.m_pictEntry.setLength(pSz);
+        database.addCell(MWAWVec2i(field.m_id, entry.id()), cell);
 #ifdef DEBUG_WITH_FILES
         ascii().skipZone(pos+4,pos+4+pSz-1);
         librevenge::RVNGBinaryData file;
@@ -1315,7 +1327,7 @@ bool GreatWksDBParser::readRowRecords(MWAWEntry const &entry)
       cell.m_content.m_contentType=MWAWCellContent::C_TEXT;
       cell.m_content.m_textEntry.setBegin(input->tell());
       cell.m_content.m_textEntry.setLength(mSz);
-      database.addCell(Vec2i(field.m_id, entry.id()), cell);
+      database.addCell(MWAWVec2i(field.m_id, entry.id()), cell);
       // probably a simple textbox ( see sendSimpleTextbox )
       if (mSz) f << "memo,";
       input->seek(pos+4+mSz, librevenge::RVNG_SEEK_SET);
@@ -1325,7 +1337,7 @@ bool GreatWksDBParser::readRowRecords(MWAWEntry const &entry)
       std::string extra("");
       ok=readFormulaResult(endPos, cell, extra);
       if (ok)
-        database.addCell(Vec2i(field.m_id, entry.id()), cell);
+        database.addCell(MWAWVec2i(field.m_id, entry.id()), cell);
       f << extra;
       break;
     }
@@ -1377,7 +1389,7 @@ bool GreatWksDBParser::readFormula(long endPos, std::vector<MWAWCellContent::For
   libmwaw::DebugStream f;
   f << "Entries(Formula):";
   std::string error("");
-  if (!m_document->readFormula(Vec2i(0,0),endHeader,formula, error))
+  if (!m_document->readFormula(MWAWVec2i(0,0),endHeader,formula, error))
     f << "###";
   f << "[";
   for (size_t l=0; l < formula.size(); ++l)
@@ -1820,7 +1832,7 @@ bool GreatWksDBParser::readFieldRecords(GreatWksDBParserInternal::Field &field)
           ++actPos;
         }
         cell.m_content.m_textEntry.setEnd(input->tell()-(findEmptyChar ? 1 : 0));
-        database.addCell(Vec2i(field.m_id, rows[i]), cell);
+        database.addCell(MWAWVec2i(field.m_id, rows[i]), cell);
         f << "\"" << text << "\":" << rows[i] << ",";
       }
       input->seek(pos+12+dSz+5*N, librevenge::RVNG_SEEK_SET);
@@ -1855,7 +1867,7 @@ bool GreatWksDBParser::readFieldRecords(GreatWksDBParserInternal::Field &field)
           cell.setFormat(field.m_format);
           cell.m_content.m_contentType=MWAWCellContent::C_NUMBER;
           cell.m_content.setValue(value);
-          database.addCell(Vec2i(field.m_id, row), cell);
+          database.addCell(MWAWVec2i(field.m_id, row), cell);
           f << value;
         }
         f << ":" << row << ",";
@@ -2137,7 +2149,7 @@ bool GreatWksDBParser::sendDatabase()
   std::vector<GreatWksDBParserInternal::Field> const &fields = database.m_fieldList;
   size_t numFields=fields.size();
   // fixme: use first layout colWidth here
-  listener->openSheet(std::vector<float>(numFields,76), librevenge::RVNG_POINT, "Sheet0");
+  listener->openSheet(std::vector<float>(1,76), librevenge::RVNG_POINT, std::vector<int>(1,(int)numFields), "Sheet0");
   int r=0, numRows=(int) database.m_rowCellsMap.size();
   std::map<int, std::vector<GreatWksDBParserInternal::Cell> >::const_iterator rIt;
   for (rIt=database.m_rowCellsMap.begin(); rIt != database.m_rowCellsMap.end(); ++rIt, ++r) {
@@ -2151,7 +2163,7 @@ bool GreatWksDBParser::sendDatabase()
       if (cell.isEmpty()) continue;
 
       MWAWCellContent const &content=cell.m_content;
-      cell.setPosition(Vec2i(int(c),r));
+      cell.setPosition(MWAWVec2i(int(c),r));
       listener->openSheetCell(cell, content);
       if (field.m_type==GreatWksDBParserInternal::Field::F_Text && content.m_textEntry.valid()) {
         input->seek(content.m_textEntry.begin(), librevenge::RVNG_SEEK_SET);
@@ -2169,6 +2181,17 @@ bool GreatWksDBParser::sendDatabase()
       }
       else if (field.m_type==GreatWksDBParserInternal::Field::F_Memo && content.m_textEntry.valid())
         m_document->getTextParser()->sendTextbox(content.m_textEntry, listener);
+      else if (field.m_type==GreatWksDBParserInternal::Field::F_Picture && cell.m_pictEntry.valid()) {
+        librevenge::RVNGBinaryData pict;
+        input->seek(cell.m_pictEntry.begin(),librevenge::RVNG_SEEK_SET);
+        input->readDataBlock(cell.m_pictEntry.length(), pict);
+
+        std::string cellName("Sheet0.");
+        cellName+=MWAWCell::getBasicCellName(cell.position()+MWAWVec2i(1,1));
+        MWAWPosition position(MWAWVec2f(0,0), MWAWVec2f(76,12), librevenge::RVNG_POINT); // changeme
+        position.setAnchorToCell(librevenge::RVNGString(cellName.c_str()));
+        listener->insertPicture(position, pict);
+      }
       listener->closeSheetCell();
     }
     listener->closeSheetRow();
@@ -2193,7 +2216,7 @@ bool GreatWksDBParser::checkHeader(MWAWHeader *header, bool strict)
   input->seek(16, librevenge::RVNG_SEEK_SET);
   for (int i=0; i<3; ++i) {
     GreatWksDBParserInternal::BlockHeader block;
-    if (!readBlockHeader(block) || block.m_ptr[0]==0 || (block.m_ptr[0]&0x0FF)) return false;
+    if (!readBlockHeader(block) || block.m_ptr[0]==0 || (block.m_ptr[0]&0xFF)) return false;
     input->seek(8, librevenge::RVNG_SEEK_CUR);
   }
   return true;
