@@ -108,6 +108,23 @@ public:
       return m_heightRows[size_t(row)];
     return m_heightDefault;
   }
+  //! returns the height of a row in point and updated repeated row
+  double getRowHeight(int row, int &numRepeated) const
+  {
+    double res=getRowHeight(row);
+    numRepeated=1;
+    if (row<0 || row>=(int) m_heightRows.size())
+      numRepeated=1000;
+    else {
+      for (size_t r=size_t(row+1); r<m_heightRows.size(); ++r) {
+        double nextH=getRowHeight(row+1);
+        if (nextH<res || nextH>res)
+          break;
+        ++numRepeated;
+      }
+    }
+    return res;
+  }
   //! convert the m_widthCols in a vector of of point size
   std::vector<float> convertInPoint(std::vector<int> const &list) const
   {
@@ -134,16 +151,16 @@ public:
   std::string m_name;
 protected:
   /** returns the last Right Bottom cell position */
-  Vec2i getRightBottomPosition() const
+  MWAWVec2i getRightBottomPosition() const
   {
     int maxX = 0, maxY = 0;
     size_t numCell = m_cells.size();
     for (size_t i = 0; i < numCell; i++) {
-      Vec2i const &p = m_cells[i].position();
+      MWAWVec2i const &p = m_cells[i].position();
       if (p[0] > maxX) maxX = p[0];
       if (p[1] > maxY) maxY = p[1];
     }
-    return Vec2i(maxX, maxY);
+    return MWAWVec2i(maxX, maxY);
   }
 };
 
@@ -213,11 +230,13 @@ void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType type
     MWAW_DEBUG_MSG(("GreatWksSSParserInternal::SubDocument::parse: unknown type\n"));
     return;
   }
-
-  assert(m_parser);
-
+  GreatWksSSParser *parser=dynamic_cast<GreatWksSSParser *>(m_parser);
+  if (!parser) {
+    MWAW_DEBUG_MSG(("GreatWksSSParserInternal::SubDocument::parse: no parser\n"));
+    return;
+  }
   long pos = m_input->tell();
-  static_cast<GreatWksSSParser *>(m_parser)->sendHF(m_id);
+  parser->sendHF(m_id);
   m_input->seek(pos, librevenge::RVNG_SEEK_SET);
 }
 
@@ -270,8 +289,7 @@ bool GreatWksSSParser::sendHF(int id)
 ////////////////////////////////////////////////////////////
 void GreatWksSSParser::parse(librevenge::RVNGSpreadsheetInterface *docInterface)
 {
-  assert(getInput().get() != 0);
-  if (!checkHeader(0L))  throw(libmwaw::ParseException());
+  if (!getInput().get() || !checkHeader(0L))  throw(libmwaw::ParseException());
   bool ok = false;
   try {
     // create the asciiFile
@@ -376,12 +394,11 @@ bool GreatWksSSParser::readSpreadsheet()
 {
   MWAWInputStreamPtr input = getInput();
   int const vers=version();
-  long pos = input->tell();
   libmwaw::DebugStream f;
 
   while (!input->isEnd()) {
     bool ok=true, printDone=false;
-    pos = input->tell();
+    long pos = input->tell();
     f.str("");
     int type=(int) input->readLong(2);
     if (type==0x10) {
@@ -712,7 +729,7 @@ bool GreatWksSSParser::readSpreadsheet()
     if (!ok && type>=7) {
       // try to use the default value
       input->seek(pos+2, librevenge::RVNG_SEEK_SET);
-      long sz=type==0x14 ? 0 : (long) input->readULong(4);
+      long sz= (long) input->readULong(4);
       endPos=pos+6+sz;
       ok=input->checkPosition(endPos);
     }
@@ -744,20 +761,27 @@ bool GreatWksSSParser::sendSpreadsheet()
   GreatWksSSParserInternal::Spreadsheet &sheet = m_state->m_spreadsheet;
   size_t numCell = sheet.m_cells.size();
 
-  listener->openSheet(sheet.convertInPoint(sheet.m_widthCols), librevenge::RVNG_POINT, sheet.m_name);
+  listener->openSheet(sheet.convertInPoint(sheet.m_widthCols), librevenge::RVNG_POINT, std::vector<int>(), sheet.m_name);
   m_document->getGraphParser()->sendPageGraphics();
 
   int prevRow = -1;
   for (size_t i = 0; i < numCell; i++) {
     GreatWksSSParserInternal::Cell cell= sheet.m_cells[i];
-    if (cell.position()[1] != prevRow) {
-      while (cell.position()[1] > prevRow) {
-        if (prevRow != -1)
-          listener->closeSheetRow();
-        prevRow++;
-        listener->openSheetRow((float)sheet.getRowHeight(prevRow), librevenge::RVNG_POINT);
+    if (cell.position()[1]>prevRow+1) {
+      while (cell.position()[1] > prevRow+1) {
+        if (prevRow != -1) listener->closeSheetRow();
+        int numRepeat;
+        double h=sheet.getRowHeight(prevRow+1, numRepeat);
+        if (cell.position()[1]<prevRow+1+numRepeat)
+          numRepeat=cell.position()[1]-1-prevRow;
+        listener->openSheetRow((float) h, librevenge::RVNG_POINT, numRepeat);
+        prevRow+=numRepeat;
       }
     }
+    if (cell.position()[1] > prevRow) {
+      if (prevRow != -1) listener->closeSheetRow();
+      listener->openSheetRow((float) sheet.getRowHeight(++prevRow), librevenge::RVNG_POINT);
+    }
 
     GreatWksSSParserInternal::Style style=m_state->getStyle(cell.m_style);
     cell.setFont(style.m_font);
@@ -850,7 +874,7 @@ bool GreatWksSSParser::readCell(GreatWksSSParserInternal::Cell &cell)
   int cPos[2];
   for (int i=0; i<2; ++i)
     cPos[i]=(int) input->readLong(2);
-  Vec2i cellPos(cPos[0]-1,cPos[1]-1);
+  MWAWVec2i cellPos(cPos[0]-1,cPos[1]-1);
   cell.setPosition(cellPos);
   f << "cell=" << cellPos << ",";
   cell.m_style=(int) input->readLong(2);
@@ -955,15 +979,15 @@ bool GreatWksSSParser::readCell(GreatWksSSParserInternal::Cell &cell)
   case 0: // none
     break;
   case 1:
-    cell.setHAlignement(MWAWCell::HALIGN_LEFT);
+    cell.setHAlignment(MWAWCell::HALIGN_LEFT);
     f << "align=left,";
     break;
   case 2:
-    cell.setHAlignement(MWAWCell::HALIGN_CENTER);
+    cell.setHAlignment(MWAWCell::HALIGN_CENTER);
     f << "align=center,";
     break;
   case 3:
-    cell.setHAlignement(MWAWCell::HALIGN_RIGHT);
+    cell.setHAlignment(MWAWCell::HALIGN_RIGHT);
     f << "align=right,";
     break;
   case 4:
@@ -1040,7 +1064,7 @@ bool GreatWksSSParser::readCell(GreatWksSSParserInternal::Cell &cell)
         f << "###";
       }
       else if (format.m_format==MWAWCell::F_DATE) // change the reference date from 1/1/1904 to 1/1/1900
-        content.setValue(value+1462.);
+        content.setValue(value+1460.);
       else
         content.setValue(value);
       f << value << ",";
@@ -1079,7 +1103,7 @@ bool GreatWksSSParser::readCell(GreatWksSSParserInternal::Cell &cell)
       }
       long value=input->readLong(4);
       if (format.m_format==MWAWCell::F_DATE) // change the reference date from 1/1/1904 to 1/1/1900
-        content.setValue(double(value)+1462.);
+        content.setValue(double(value)+1460.);
       else
         content.setValue(double(value));
       f << value << ",";
@@ -1217,7 +1241,7 @@ bool GreatWksSSParser::readStyles()
     else {
       f << "pattern[id]=" << patId << ",";
       MWAWGraphicStyle::Pattern pattern;
-      pattern.m_dim=Vec2i(8,8);
+      pattern.m_dim=MWAWVec2i(8,8);
       pattern.m_data.resize(8);
       for (size_t j=0; j < 8; ++j)
         pattern.m_data[j]=(unsigned char) input->readULong(1);
