@@ -143,10 +143,14 @@ void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType type
     MWAW_DEBUG_MSG(("HanMacWrdJParserInternal::SubDocument::parse: unexpected document type\n"));
     return;
   }
+  HanMacWrdJParser *parser=dynamic_cast<HanMacWrdJParser *>(m_parser);
+  if (!parser) {
+    MWAW_DEBUG_MSG(("HanMacWrdJParserInternal::SubDocument::parse: no parser\n"));
+    return;
+  }
 
-  assert(m_parser);
   long pos = m_input->tell();
-  static_cast<HanMacWrdJParser *>(m_parser)->sendText(m_id, 0);
+  parser->sendText(m_id, 0);
   m_input->seek(pos, librevenge::RVNG_SEEK_SET);
 }
 }
@@ -190,7 +194,7 @@ bool HanMacWrdJParser::canSendTextAsGraphic(long id, long cPos)
 
 bool HanMacWrdJParser::sendZone(long zId)
 {
-  MWAWPosition pos(Vec2i(0,0), Vec2i(0,0), librevenge::RVNG_POINT);
+  MWAWPosition pos(MWAWVec2i(0,0), MWAWVec2i(0,0), librevenge::RVNG_POINT);
   pos.setRelativePosition(MWAWPosition::Char);
   return m_graphParser->sendFrame(zId, pos);
 }
@@ -203,10 +207,10 @@ bool HanMacWrdJParser::getColor(int colId, int patternId, MWAWColor &color) cons
 ////////////////////////////////////////////////////////////
 // position and height
 ////////////////////////////////////////////////////////////
-Vec2f HanMacWrdJParser::getPageLeftTop() const
+MWAWVec2f HanMacWrdJParser::getPageLeftTop() const
 {
-  return Vec2f(float(getPageSpan().getMarginLeft()),
-               float(getPageSpan().getMarginTop()+m_state->m_headerHeight/72.0));
+  return MWAWVec2f(float(getPageSpan().getMarginLeft()),
+                   float(getPageSpan().getMarginTop()+m_state->m_headerHeight/72.0));
 }
 
 ////////////////////////////////////////////////////////////
@@ -251,9 +255,7 @@ bool HanMacWrdJParser::readClassicHeader(HanMacWrdJZoneHeader &header, long endP
 ////////////////////////////////////////////////////////////
 void HanMacWrdJParser::parse(librevenge::RVNGTextInterface *docInterface)
 {
-  assert(getInput().get() != 0);
-
-  if (!checkHeader(0L)) throw(libmwaw::ParseException());
+  if (!getInput().get() || !checkHeader(0L)) throw(libmwaw::ParseException());
   bool ok = true;
   try {
     // create the asciiFile
@@ -438,7 +440,7 @@ bool HanMacWrdJParser::checkEntry(MWAWEntry &entry)
     "TextZone", "Picture", "Table", "GraphData", "GroupData",
     "ZoneA", "ZoneB", "Section", "FtnDef", "ZoneE", "FontsName"
   };
-  if (type <= 15)
+  if (type>=0 && type <= 15)
     entry.setName(what[type]);
   else {
     std::stringstream s;
@@ -645,27 +647,27 @@ bool HanMacWrdJParser::readPrintInfo(MWAWEntry const &entry)
     return false;
   }
 
-  Vec2i paperSize = info.paper().size();
-  Vec2i pageSize = info.page().size();
+  MWAWVec2i paperSize = info.paper().size();
+  MWAWVec2i pageSize = info.page().size();
 
   bool useDocInfo = (dim[3]-dim[1]>margins[2]+margins[0]) &&
                     (dim[2]-dim[0]>margins[2]+margins[0]);
   bool usePrintInfo = pageSize.x() > 0 && pageSize.y() > 0 &&
                       paperSize.x() > 0 && paperSize.y() > 0;
 
-  Vec2f lTopMargin(margins[0],margins[1]), rBotMargin(margins[2],margins[3]);
+  MWAWVec2f lTopMargin(margins[0],margins[1]), rBotMargin(margins[2],margins[3]);
   // define margin from print info
   if (useDocInfo)
-    paperSize = Vec2i(dim[3]-dim[1],dim[2]-dim[0]);
+    paperSize = MWAWVec2i(dim[3]-dim[1],dim[2]-dim[0]);
   else if (usePrintInfo) {
-    lTopMargin= Vec2f(-float(info.paper().pos(0)[0]), -float(info.paper().pos(0)[1]));
+    lTopMargin= MWAWVec2f(-float(info.paper().pos(0)[0]), -float(info.paper().pos(0)[1]));
     rBotMargin=info.paper().pos(1) - info.page().pos(1);
 
     // move margin left | top
     float decalX = lTopMargin.x() > 14 ? 14 : 0;
     float decalY = lTopMargin.y() > 14 ? 14 : 0;
-    lTopMargin -= Vec2f(decalX, decalY);
-    rBotMargin += Vec2f(decalX, decalY);
+    lTopMargin -= MWAWVec2f(decalX, decalY);
+    rBotMargin += MWAWVec2f(decalX, decalY);
   }
 
   // decrease right | bottom
@@ -1199,7 +1201,7 @@ bool HanMacWrdJParser::decodeZone(MWAWEntry const &entry, librevenge::RVNGBinary
     do {  /* once for each bit on path */
       if (bitcounter == 0) {
         if (input->isEnd() || input->tell() >= entry.end()) {
-          MWAW_DEBUG_MSG(("HanMacWrdJParser::decodeZone: find some uncomplete data for zone%lx\n", entry.begin()));
+          MWAW_DEBUG_MSG(("HanMacWrdJParser::decodeZone: find some uncomplete data for zone %lx\n", (long unsigned int) entry.begin()));
           dt.append((unsigned char)a);
           ok = false;
           break;
