@@ -127,6 +127,18 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
         return res;
       }
     }
+    else if (creator=="CDrw") { // CHANGEME: ClarisDraw
+      if (type=="dDrw" || type=="dDst" || type=="iLib") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CLARISDRAW, 1, MWAWDocument::MWAW_K_DRAW));
+        return res;
+      }
+    }
+    else if (creator=="C#+A") { // solo
+      if (type=="C#+D" || type=="C#+F") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_RAGTIME, 5));
+        return res;
+      }
+    }
     else if (creator=="Dc@P" || creator=="Dk@P") {
       if (type=="APPL") {
         res.push_back(MWAWHeader(MWAWDocument::MWAW_T_DOCMAKER, 1));
@@ -189,6 +201,40 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
         return res;
       }
     }
+    else if (creator=="MACD") { // v1.0
+      if (type=="DRWG") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MACDRAFT, 1, MWAWDocument::MWAW_K_DRAW));
+        return res;
+      }
+    }
+    else if (creator=="MD40") {
+      if (type=="MDDC" || type=="MSYM") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MACDRAFT, 4, MWAWDocument::MWAW_K_DRAW));
+        return res;
+      }
+    }
+    else if (creator=="MDFT") { // v1.2
+      if (type=="DRWG") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MACDRAFT, 1, MWAWDocument::MWAW_K_DRAW));
+        return res;
+      }
+    }
+    else if (creator=="MDPL") { // MacDraw II
+      if (type=="DRWG") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MACDRAWPRO, 0, MWAWDocument::MWAW_K_DRAW));
+        return res;
+      }
+      if (type=="STAT") { // stationery
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MACDRAWPRO, 0, MWAWDocument::MWAW_K_DRAW));
+        return res;
+      }
+    }
+    else if (creator=="MDRW") {
+      if (type=="DRWG") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MACDRAW, 1, MWAWDocument::MWAW_K_DRAW));
+        return res;
+      }
+    }
     else if (creator=="MDsr") {
       if (type=="APPL") { // auto content
         res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MACDOC, 1));
@@ -280,6 +326,18 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
       }
       // "edtt": empty file, probably created when the file is edited
     }
+    else if (creator=="PANT") {
+      if (type=="PNTG") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_FULLPAINT, 1, MWAWDocument::MWAW_K_PAINT));
+        return res;
+      }
+    }
+    else if (creator=="PIXR") {
+      if (type=="PX01") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_PIXELPAINT, 1, MWAWDocument::MWAW_K_DRAW));
+        return res;
+      }
+    }
     else if (creator=="PSIP") {
       if (type=="AWWP") {
         res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MICROSOFTWORKS, 1));
@@ -330,6 +388,19 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
       }
       // other type seems to correspond to basic picture file, so we do not accept them
     }
+    else if (creator=="StAV") {
+      if (type=="APPL") { // Style: document application
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_STYLE, 1));
+        return res;
+      }
+    }
+    else if (creator==std::string("St")+(char) 0xd8+"l") { //  argh, not standart character
+      std::string type1=std::string("TEd")+(char)0xb6; // argh, not standart character
+      if (type==type1) {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_STYLE, 1));
+        return res;
+      }
+    }
     else if (creator=="TBB5") {
       if (type=="TEXT" || type=="ttro") {
         res.push_back(MWAWHeader(MWAWDocument::MWAW_T_TEXEDIT, 1));
@@ -384,6 +455,16 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
         return res;
       }
     }
+    else if (creator=="dPro") {
+      if (type=="dDoc") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MACDRAWPRO, 1, MWAWDocument::MWAW_K_DRAW));
+        return res;
+      }
+      if (type=="dLib") { // macdraw pro slide/library
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MACDRAWPRO, 1, MWAWDocument::MWAW_K_DRAW));
+        return res;
+      }
+    }
     else if (creator=="eDcR") {
       if (type=="eDoc") {
         res.push_back(MWAWHeader(MWAWDocument::MWAW_T_EDOC, 1));
@@ -412,6 +493,10 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
         return res;
       }
     }
+    else if (type=="PICT") {
+      res.push_back(MWAWHeader(MWAWDocument::MWAW_T_APPLEPICT, 1, MWAWDocument::MWAW_K_DRAW));
+      return res;
+    }
     MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: unknown finder info: type=%s[%s]\n", type.c_str(), creator.c_str()));
 
   }
@@ -422,15 +507,15 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
     return res;
 
   input->seek(0, librevenge::RVNG_SEEK_SET);
-  int val[4];
-  for (int i = 0; i < 4; i++)
+  int val[5];
+  for (int i = 0; i < 5; i++)
     val[i] = int(input->readULong(2));
 
   // ----------- clearly discriminant ------------------
   if (val[2] == 0x424F && val[3] == 0x424F && (val[0]>>8) < 7) {
     MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a Claris Works file\n"));
     int vers= (val[0] >> 8);
-    static int const(typePos[7])= {0, 243, 249, 249, 256, 268, 278};
+    static int const(typePos[7])= {0, 242, 248, 248, 256, 268, 278};
     int typeFile=-1;
     if (vers >= 1 && vers <= 6 && input->checkPosition(typePos[vers])) {
       input->seek(typePos[vers], librevenge::RVNG_SEEK_SET);
@@ -488,6 +573,11 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
     res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CLARISRESOLVE, 1, MWAWDocument::MWAW_K_SPREADSHEET));
     return res;
   }
+  if (val[0]==0x4323 && val[1]==0x2b44 && val[2]==0xa443 && val[3]==0x4da5) {
+    MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a RagTime 5-6 file\n"));
+    res.push_back(MWAWHeader(MWAWDocument::MWAW_T_RAGTIME, 5));
+    return res;
+  }
   if (val[0]==0x4646 && val[1]==0x4646 && val[2]==0x3030 && val[3]==0x3030) {
     MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a Mariner Write file\n"));
     res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MARINERWRITE, 1));
@@ -520,6 +610,41 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
       return res;
     }
   }
+  if (val[0]==0x4452 && val[1]==0x5747) { // DRWG
+    if (val[2]==0x4d44) { // MD
+      MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a MacDraw file\n"));
+      res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MACDRAW, 1, MWAWDocument::MWAW_K_DRAW));
+      return res;
+    }
+    if (val[2]==0 || val[2]==0x4432) { // D2
+      MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a MacDraw II file\n"));
+      res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MACDRAWPRO, 0, MWAWDocument::MWAW_K_DRAW));
+      return res;
+    }
+  }
+  if (val[0]==0x5354 && val[1]==0x4154 && (val[2]==0 || val[2]==0x4432)) {
+    MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a MacDraw II template file\n"));
+    res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MACDRAWPRO, 0, MWAWDocument::MWAW_K_DRAW));
+    return res;
+  }
+#ifdef DEBUG
+  // we need the resource fork to find the colors, patterns, ... ; so not active in normal mode
+  if (val[0]==0x6444 && val[1]==0x6f63 && val[2]==0x4432) { // dDocD2
+    MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a MacDraw Pro file\n"));
+    res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MACDRAWPRO, 1, MWAWDocument::MWAW_K_DRAW));
+    return res;
+  }
+  if (val[0]==0x644c && val[1]==0x6962 && val[2]==0x4432) { // dLibD2
+    MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a MacDraw Pro template file\n"));
+    res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MACDRAWPRO, 1, MWAWDocument::MWAW_K_DRAW));
+    return res;
+  }
+#endif
+  if (val[0]==2 && val[1]==0 && val[2]==2 && val[3]==0x262 && val[4]==0x262) {
+    MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a MacDraft file\n"));
+    res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MACDRAFT, 1, MWAWDocument::MWAW_K_DRAW));
+    return res;
+  }
   if (val[0]==0x4859 && val[1]==0x4c53 && val[2]==0x0210) {
     MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a HanMac Word-K file\n"));
     res.push_back(MWAWHeader(MWAWDocument::MWAW_T_HANMACWORDK, 1));
@@ -538,6 +663,10 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
     res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MORE, 3));
     return res;
   }
+  if ((val[0]==0x100||val[0]==0x200) && val[2]==0x4558 && val[3]==0x5057) { // CHANGEME: ClarisDraw
+    res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CLARISDRAW, 1, MWAWDocument::MWAW_K_DRAW));
+    return res;
+  }
 
   if (val[0]==0x100 || val[0]==0x200) {
     if (val[1]==0x5a57 && val[2]==0x5254) {
@@ -558,6 +687,15 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
     }
     // maybe we can also add outline: if (val[1]==0x5a4f && val[2]==0x4c4e)
   }
+  if (val[3]==6 && val[4]<6) {
+    if (val[0]==0x4d44 && val[1]==0x4443 && val[2]==0x3230) {
+      res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MACDRAFT, 4, MWAWDocument::MWAW_K_DRAW));
+      return res;
+    }
+    // can be a library file, this will be test in the parser
+    if (input->size()>=30)
+      res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MACDRAFT, 4, MWAWDocument::MWAW_K_DRAW));
+  }
   // magic ole header
   if (val[0]==0xd0cf && val[1]==0x11e0 && val[2]==0xa1b1 && val[3]==0x1ae1 && input->isStructured()) {
     MWAWInputStreamPtr mainOle = input->getSubStreamByName("MN0");
@@ -610,6 +748,10 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
     if (vers >= 0)
       res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MICROSOFTWORD, vers));
   }
+  if (val[0]==0x4348 && val[1]==0x4e4b && val[2]==0x100 && val[3]==0) {
+    MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a Style file\n"));
+    res.push_back(MWAWHeader(MWAWDocument::MWAW_T_STYLE, 1));
+  }
 
   // ----------- less discriminant ------------------
   if (val[0] == 0x2e && val[1] == 0x2e) {
@@ -680,6 +822,9 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
       }
     }
   }
+  if (val[0]==0x4d44 && input->size()>=512) { // maybe a MacDraw 0 file, will be check later
+    res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MACDRAW, 0, MWAWDocument::MWAW_K_DRAW));
+  }
   if (val[0] == 3 || val[0] == 6) {
     // version will be print by MacWrtParser::check
     res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MACWRITE, val[0]));
@@ -697,6 +842,27 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
     else if (value==2)
       res.push_back(MWAWHeader(MWAWDocument::MWAW_T_SUPERPAINT, 1, MWAWDocument::MWAW_K_DRAW));
   }
+  if (val[0]==0 && (val[1]==0x7FFF || val[1]==0x8000)) {
+    MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a potential PixelPaint file\n"));
+    res.push_back(MWAWHeader(MWAWDocument::MWAW_T_PIXELPAINT, (val[1]==0x7FFF) ? 1  : 2, MWAWDocument::MWAW_K_PAINT));
+  }
+  //
+  // check for pict
+  //
+  for (int st=0; st<2; ++st) {
+    if (!input->checkPosition(512*st+13))
+      break;
+    input->seek(512*st+10, librevenge::RVNG_SEEK_SET);
+    int value=(int) input->readULong(2);
+    if (value==0x1101) {
+      res.push_back(MWAWHeader(MWAWDocument::MWAW_T_APPLEPICT, 1, MWAWDocument::MWAW_K_DRAW));
+      break;
+    }
+    else if (value==0x11 && input->readULong(2)==0x2ff && input->readULong(2) == 0xC00) {
+      res.push_back(MWAWHeader(MWAWDocument::MWAW_T_APPLEPICT, 2, MWAWDocument::MWAW_K_DRAW));
+      break;
+    }
+  }
   //
   // middle of file
   //
@@ -736,7 +902,7 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
       if (!ok) break;
     }
     if (ok) {
-      MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a MacPaint file\n"));
+      MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a potential MacPaint file\n"));
       res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MACPAINT, 1, MWAWDocument::MWAW_K_PAINT));
     }
   }
