@@ -61,7 +61,7 @@ namespace MacWrtProParserInternal
 ////////////////////////////////////////
 //! Internal: a struct used to store a zone
 struct Zone {
-  Zone() : m_type(-1), m_blockId(0), m_data(), m_input(), m_asciiFile(), m_parsed(false)
+  Zone() : m_type(-1), m_blockId(0), m_data(), m_input(), m_asciiFile(MWAWInputStreamPtr()), m_parsed(false)
   {
   }
   ~Zone()
@@ -180,7 +180,7 @@ struct Token {
   //! the block id
   int m_blockId;
   //! the bdbox ( filled in MWII for figure)
-  Box2f m_box;
+  MWAWBox2f m_box;
   //! some flags
   int m_flags[4];
 };
@@ -279,13 +279,16 @@ void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType /*ty
 {
   if (m_id == -3) return; // empty block
   if (!listener.get()) {
-    MWAW_DEBUG_MSG(("SubDocument::parse: no listener\n"));
+    MWAW_DEBUG_MSG(("MacWrtProParserInternal::SubDocument::parse: no listener\n"));
+    return;
+  }
+  MacWrtProParser *parser = dynamic_cast<MacWrtProParser *>(m_parser);
+  if (!parser) {
+    MWAW_DEBUG_MSG(("MacWrtProParserInternal::SubDocument::parse: no parser\n"));
     return;
   }
-  assert(m_parser);
 
   long pos = m_input->tell();
-  MacWrtProParser *parser = static_cast<MacWrtProParser *>(m_parser);
   if (parser->m_structures.get())
     parser->m_structures->send(m_id);
   m_input->seek(pos, librevenge::RVNG_SEEK_SET);
@@ -369,9 +372,7 @@ std::vector<int> const &MacWrtProParser::getBlocksCalledByToken() const
 ////////////////////////////////////////////////////////////
 void MacWrtProParser::parse(librevenge::RVNGTextInterface *docInterface)
 {
-  assert(getInput().get() != 0);
-
-  if (!checkHeader(0L))  throw(libmwaw::ParseException());
+  if (!getInput().get() || !checkHeader(0L))  throw(libmwaw::ParseException());
   bool ok = true;
   try {
     m_state->m_blocksMap.clear();
@@ -525,7 +526,6 @@ bool MacWrtProParser::getFreeZoneList(int blockId, std::vector<int> &blockLists)
     input->seek(endPos, librevenge::RVNG_SEEK_SET);
     if (long(input->tell()) != endPos) break;
 
-
     long act = last;
     long val = input->readLong(4);
     if (val == 0) break;
@@ -746,20 +746,20 @@ bool MacWrtProParser::readPrintInfo()
   if (!info.read(input)) return false;
   f << "Entries(PrintInfo):"<< info;
 
-  Vec2i paperSize = info.paper().size();
-  Vec2i pageSize = info.page().size();
+  MWAWVec2i paperSize = info.paper().size();
+  MWAWVec2i pageSize = info.page().size();
   if (pageSize.x() <= 0 || pageSize.y() <= 0 ||
       paperSize.x() <= 0 || paperSize.y() <= 0) return false;
 
   // define margin from print info
-  Vec2i lTopMargin= -1 * info.paper().pos(0);
-  Vec2i rBotMargin=info.paper().pos(1) - info.page().pos(1);
+  MWAWVec2i lTopMargin= -1 * info.paper().pos(0);
+  MWAWVec2i rBotMargin=info.paper().pos(1) - info.page().pos(1);
 
   // move margin left | top
   int decalX = lTopMargin.x() > 14 ? lTopMargin.x()-14 : 0;
   int decalY = lTopMargin.y() > 14 ? lTopMargin.y()-14 : 0;
-  lTopMargin -= Vec2i(decalX, decalY);
-  rBotMargin += Vec2i(decalX, decalY);
+  lTopMargin -= MWAWVec2i(decalX, decalY);
+  rBotMargin += MWAWVec2i(decalX, decalY);
 
   // decrease right | bottom
   int rightMarg = rBotMargin.x() -10;
@@ -1315,7 +1315,7 @@ bool MacWrtProParser::readTextTokens(shared_ptr<MacWrtProParserInternal::Zone> z
       f << "sz=" << std::hex << sz << std::dec << ",";
       int dim[4];
       for (int j = 0; j < 4; j++) dim[j] = (int) input->readLong(2);
-      token.m_box = Box2f(Vec2f(float(dim[1]),float(dim[0])), Vec2f(float(dim[3]),float(dim[2])));
+      token.m_box = MWAWBox2f(MWAWVec2f(float(dim[1]),float(dim[0])), MWAWVec2f(float(dim[3]),float(dim[2])));
       f << "dim=[" << dim[1] << "x" << dim[0] << "-"
         << dim[3] << "x" << dim[2] << ",";
       for (int j = 0; j < 4; j++) dim[j] = (int) input->readLong(2);
@@ -1550,7 +1550,7 @@ bool MacWrtProParser::sendText(shared_ptr<MacWrtProParserInternal::TextZone> zon
         break; // footnote content, ok
       case 4:
         if (vers==0) {
-          MWAWPosition pictPos(Vec2i(0,0), zone->m_tokens[(size_t)data.m_id].m_box.size(), librevenge::RVNG_POINT);
+          MWAWPosition pictPos(MWAWVec2i(0,0), zone->m_tokens[(size_t)data.m_id].m_box.size(), librevenge::RVNG_POINT);
           pictPos.setRelativePosition(MWAWPosition::Char, MWAWPosition::XLeft, MWAWPosition::YBottom);
           sendPictureZone(zone->m_tokens[(size_t)data.m_id].m_blockId, pictPos);
         }
@@ -1571,14 +1571,14 @@ bool MacWrtProParser::sendText(shared_ptr<MacWrtProParserInternal::TextZone> zon
         if (getTextListener()) getTextListener()->insertField(MWAWField(MWAWField::Title));
         break;
       case 9:
-        if (getTextListener()) getTextListener()->insertUnicodeString("#REVISION#");
+        if (getTextListener()) getTextListener()->insertUnicodeString(librevenge::RVNGString("#REVISION#"));
         break;
       case 10:
         if (getTextListener()) {
           int numSection = listenerState.numSection()+1;
           std::stringstream s;
           s << numSection;
-          getTextListener()->insertUnicodeString(s.str().c_str());
+          getTextListener()->insertUnicodeString(librevenge::RVNGString(s.str().c_str()));
         }
         break;
       default:
@@ -1711,18 +1711,18 @@ bool MacWrtProParser::sendPicture(shared_ptr<MacWrtProParserInternal::Zone> zone
   if (!pict) { // ok, we can not do anything except sending the data...
     MWAW_DEBUG_MSG(("MacWrtProParser::parseDataZone: no sure this is a picture\n"));
     if (pictPos.size().x() <= 0 || pictPos.size().y() <= 0)
-      pictPos=MWAWPosition(Vec2f(0,0),Vec2f(100.,100.), librevenge::RVNG_POINT);
+      pictPos=MWAWPosition(MWAWVec2f(0,0),MWAWVec2f(100.,100.), librevenge::RVNG_POINT);
     if (getTextListener()) {
       librevenge::RVNGBinaryData data;
       input->seek(4, librevenge::RVNG_SEEK_SET);
       input->readDataBlock(pictSize, data);
-      getTextListener()->insertPicture(pictPos, data, "image/pict", style);
+      getTextListener()->insertPicture(pictPos, MWAWEmbeddedObject(data, "image/pict"), style);
     }
     return true;
   }
 
   if (pictPos.size().x() <= 0 || pictPos.size().y() <= 0) {
-    pictPos.setOrigin(Vec2f(0,0));
+    pictPos.setOrigin(MWAWVec2f(0,0));
     pictPos.setSize(pict->getBdBox().size());
     pictPos.setUnit(librevenge::RVNG_POINT);
   }
@@ -1730,10 +1730,9 @@ bool MacWrtProParser::sendPicture(shared_ptr<MacWrtProParserInternal::Zone> zone
     pictPos.setNaturalSize(pict->getBdBox().size());
 
   if (getTextListener()) {
-    librevenge::RVNGBinaryData data;
-    std::string type;
-    if (pict->getBinary(data,type))
-      getTextListener()->insertPicture(pictPos, data, type, style);
+    MWAWEmbeddedObject picture;
+    if (pict->getBinary(picture))
+      getTextListener()->insertPicture(pictPos, picture, style);
   }
   return true;
 }
@@ -1748,8 +1747,7 @@ void MacWrtProParser::saveOriginal(MWAWInputStreamPtr input)
   libmwaw::DebugStream f;
 
   int ptSz = version()==0 ? 2 : 4;
-  libmwaw::DebugFile orig;
-  orig.setStream(input);
+  libmwaw::DebugFile orig(input);
   orig.open("orig");
   int bl = 0;
   while (1) {
