@@ -202,7 +202,7 @@ struct Block {
   {
     MWAWPosition res;
     if (m_attachment) {
-      res = MWAWPosition(Vec2i(0,0), m_box.size(), librevenge::RVNG_POINT);
+      res = MWAWPosition(MWAWVec2i(0,0), m_box.size(), librevenge::RVNG_POINT);
       res.setRelativePosition(MWAWPosition::Char, MWAWPosition::XLeft, getRelativeYPos());
     }
     else {
@@ -222,12 +222,12 @@ struct Block {
     if (m_baseline < 0.75*height) return MWAWPosition::YCenter;
     return MWAWPosition::YTop;
   }
-  bool contains(Box2f const &box) const
+  bool contains(MWAWBox2f const &box) const
   {
     return box[0][0] >= m_box[0][0] && box[0][1] >= m_box[0][1] &&
            box[1][0] <= m_box[1][0] && box[1][1] <= m_box[1][1];
   }
-  bool intersects(Box2f const &box) const
+  bool intersects(MWAWBox2f const &box) const
   {
     if (box[0][0] >= m_box[1][0] || box[0][1] >= m_box[1][1] ||
         box[1][0] <= m_box[0][0] || box[1][1] <= m_box[1][1])
@@ -257,7 +257,7 @@ struct Block {
   int m_page;
 
   //! the bdbox
-  Box2f m_box;
+  MWAWBox2f m_box;
 
   //! the borders width
   double m_borderWList[4];
@@ -357,7 +357,7 @@ struct Cell : public MWAWCell {
   Cell(MacWrtProStructures &parser, Block *block) : MWAWCell(), m_parser(parser), m_blockId(0)
   {
     if (!block) return;
-    setBdBox(Box2f(block->m_box.min(), block->m_box.max()-Vec2f(1,1)));
+    setBdBox(MWAWBox2f(block->m_box.min(), block->m_box.max()-MWAWVec2f(1,1)));
     setBackgroundColor(block->m_surfaceColor);
     m_blockId = block->m_id;
     for (int b=0; b<4; ++b) {
@@ -559,7 +559,7 @@ struct State {
 ////////////////////////////////////////////////////////////
 MacWrtProStructures::MacWrtProStructures(MacWrtProParser &parser) :
   m_parserState(parser.getParserState()), m_input(), m_mainParser(parser),
-  m_state(), m_asciiFile(), m_asciiName("")
+  m_state(), m_asciiFile(MWAWInputStreamPtr()), m_asciiName("")
 {
   init();
 }
@@ -1142,11 +1142,6 @@ bool MacWrtProStructures::readFontsName()
     return false;
   }
   m_input->seek(pos+4, librevenge::RVNG_SEEK_SET);
-  if (sz == 0) {
-    ascii().addPos(pos);
-    ascii().addNote("_");
-    return true;
-  }
   f << "Entries(FontsName):";
   int N=(int) m_input->readULong(2);
   if (3*N+2 > sz) {
@@ -1271,7 +1266,7 @@ bool MacWrtProStructures::readFont(MacWrtProStructuresInternal::Font &font)
   if (flag&0x40) font.m_font.set(MWAWFont::Script(-40,librevenge::RVNG_PERCENT));
   if (flag&0x100) font.m_font.set(MWAWFont::Script::super());
   if (flag&0x200) font.m_font.setStrikeOutStyle(MWAWFont::Line::Simple);
-  if (flag&0x400) flags |= MWAWFont::allCapsBit;
+  if (flag&0x400) flags |= MWAWFont::uppercaseBit;
   if (flag&0x800) flags |= MWAWFont::smallCapsBit;
   if (flag&0x1000) font.m_font.setUnderlineStyle(MWAWFont::Line::Simple);
   if (flag&0x2000) {
@@ -1470,7 +1465,7 @@ bool MacWrtProStructures::readParagraph(MacWrtProStructuresInternal::Paragraph &
     ascii().addDelimiter(m_input->tell(),'|');
   }
   /* Note: when no extra tab the justification,
-           if there is a extra tab, this corresponds to the extra tab alignement :-~ */
+           if there is a extra tab, this corresponds to the extra tab alignment :-~ */
   switch (just & 0x3) {
   case 0:
     break;
@@ -1881,12 +1876,15 @@ shared_ptr<MacWrtProStructuresInternal::Block>  MacWrtProStructures::readBlockV2
     if (val) f<< "unkn=" << val << ",";
     int what = (int) m_input->readULong(1);
     switch (what &0xF0) {
-    case 0x40:
+    case 0x40: // a header zone
       res->m_isHeader = true;
-    case 0x80:
       res->m_type = 6;
       break;
-    case 0xc0:
+    case 0x80: // a footer zone
+      res->m_isHeader = false;
+      res->m_type = 6;
+      break;
+    case 0xc0: // a footnote zone
       res->m_type = 7;
       break;
     default:
@@ -1950,7 +1948,7 @@ shared_ptr<MacWrtProStructuresInternal::Block>  MacWrtProStructures::readBlockV2
   float dim[4];
   for (int i = 0; i < 4; ++i)
     dim[i] = float(m_input->readLong(2));
-  res->m_box = Box2f(Vec2f(dim[1],dim[0]), Vec2f(dim[3],dim[2]));
+  res->m_box = MWAWBox2f(MWAWVec2f(dim[1],dim[0]), MWAWVec2f(dim[3],dim[2]));
   for (int i = 0; i < 4; ++i) { // 8000*4 ?
     val = (long) m_input->readULong(i==3 ? 1 : 2);
     if (val != 0x8000) f << "g" << i+1 << "=" << std::hex << val << std::dec << ",";
@@ -2015,7 +2013,7 @@ shared_ptr<MacWrtProStructuresInternal::Block> MacWrtProStructures::readBlock()
   float dim[4];
   for (int i = 0; i < 4; ++i)
     dim[i] = float(m_input->readLong(4))/65536.f;
-  block->m_box = Box2f(Vec2f(dim[1],dim[0]), Vec2f(dim[3],dim[2]));
+  block->m_box = MWAWBox2f(MWAWVec2f(dim[1],dim[0]), MWAWVec2f(dim[3],dim[2]));
 
   static int const(wh[4])= { libmwaw::Top, libmwaw::Left, libmwaw::Bottom, libmwaw::Right };
   for (int i = 0; i < 4; ++i)
@@ -2113,7 +2111,7 @@ shared_ptr<MacWrtProStructuresInternal::Block> MacWrtProStructures::readBlock()
       if (isNote && val) {
         // ok, reset the box only if it is bigger
         if (dim2[3]-dim2[1]>dim[3]-dim[1] && dim2[2]-dim2[0]>dim[2]-dim[0])
-          block->m_box=Box2f(Vec2f(dim2[1],dim2[0]),Vec2f(dim2[3],dim2[2]));
+          block->m_box=MWAWBox2f(MWAWVec2f(dim2[1],dim2[0]),MWAWVec2f(dim2[3],dim2[2]));
       }
     }
     if (isNote) {
@@ -2276,6 +2274,10 @@ bool MacWrtProStructures::readSections(std::vector<MacWrtProStructuresInternal::
     m_input->seek(endPos, librevenge::RVNG_SEEK_SET);
     return true;
   }
+  if (!m_input->checkPosition(endPos)) {
+    MWAW_DEBUG_MSG(("MacWrtProStructures::readSections: section is outside of the input\n"));
+    return true;
+  }
 
   int N = int(sz/0xd8);
   f << "Entries(Section):";
@@ -2426,13 +2428,11 @@ bool MacWrtProStructures::readString(MWAWInputStreamPtr input, std::string &res)
     MWAW_DEBUG_MSG(("MacWrtProStructures::readString: odd value for size\n"));
     return false;
   }
-  input->seek(pos+sz+2, librevenge::RVNG_SEEK_SET);
-  if (long(input->tell())!=pos+sz+2) {
+  if (!input->checkPosition(pos+sz+2)) {
     input->seek(pos, librevenge::RVNG_SEEK_SET);
     MWAW_DEBUG_MSG(("MacWrtProStructures::readString: file is too short\n"));
     return false;
   }
-  input->seek(pos+2, librevenge::RVNG_SEEK_SET);
   for (int i= 0; i < sz; ++i) {
     char c = (char) input->readULong(1);
     if (c) {
@@ -2465,13 +2465,11 @@ bool MacWrtProStructures::readStructB()
 
   // CHECKME: find N=2 only one time ( and across a checksum zone ...)
   long endPos = pos+N*10+6;
-  m_input->seek(endPos, librevenge::RVNG_SEEK_SET);
-  if (long(m_input->tell()) != endPos) {
+  if (!m_input->checkPosition(endPos)) {
     MWAW_DEBUG_MSG(("MacWrtProStructures::readZonB: file is too short\n"));
     m_input->seek(pos, librevenge::RVNG_SEEK_SET);
     return false;
   }
-  m_input->seek(pos+2, librevenge::RVNG_SEEK_SET);
   int val = (int) m_input->readULong(2);
   if (val != 0x2af8)
     f << "f0=" << std::hex << val << std::dec << ",";
