@@ -67,7 +67,7 @@ enum PLCType { FONT=0, RULER, FOOTNOTE, PAGE, ZONE, UNKNOWN};
 /** Internal: class to store the PLC: Pointer List Content ? */
 struct PLC {
   //! constructor
-  PLC(PLCType type=UNKNOWN): m_type(type), m_id(-1), m_extras("") {}
+  explicit PLC(PLCType type=UNKNOWN): m_type(type), m_id(-1), m_extras("") {}
   //! operator<<
   friend std::ostream &operator<<(std::ostream &o, PLC const &plc);
   //! the type
@@ -188,7 +188,7 @@ struct State {
   //! the zones limits
   int m_fileZonesLimit[7];
   //! the list of text zones
-  std::vector<Vec2l> m_textZonesList;
+  std::vector<MWAWVec2l> m_textZonesList;
   //! the list of main text zones
   std::vector<int> m_mainTextZonesList;
   //! the list of fonts
@@ -198,7 +198,7 @@ struct State {
   //! a flag to know if we send endnote or footnote
   bool m_endNote;
   //! the footnote positions ( list of beginPod, endPos)
-  std::vector<Vec2l> m_footnotesList;
+  std::vector<MWAWVec2l> m_footnotesList;
   //! the text correspondance zone ( filepos, plc )
   std::multimap<long, PLC> m_plcMap;
 
@@ -243,17 +243,21 @@ protected:
 void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType)
 {
   if (!listener.get()) {
-    MWAW_DEBUG_MSG(("SubDocument::parse: no listener\n"));
+    MWAW_DEBUG_MSG(("MsWrd1ParserInternal::SubDocument::parse: no listener\n"));
+    return;
+  }
+  MsWrd1Parser *parser=dynamic_cast<MsWrd1Parser *>(m_parser);
+  if (!parser) {
+    MWAW_DEBUG_MSG(("MsWrd1ParserInternal::SubDocument::parse: no parser\n"));
     return;
   }
-  assert(m_parser);
 
   if (!m_zone.valid()) {
     listener->insertChar(' ');
     return;
   }
   long pos = m_input->tell();
-  static_cast<MsWrd1Parser *>(m_parser)->sendText(m_zone);
+  parser->sendText(m_zone);
   m_input->seek(pos, librevenge::RVNG_SEEK_SET);
 }
 }
@@ -314,9 +318,7 @@ void MsWrd1Parser::removeLastCharIfEOL(MWAWEntry &entry)
 ////////////////////////////////////////////////////////////
 void MsWrd1Parser::parse(librevenge::RVNGTextInterface *docInterface)
 {
-  assert(getInput().get() != 0);
-
-  if (!checkHeader(0L))  throw(libmwaw::ParseException());
+  if (!getInput().get() || !checkHeader(0L))  throw(libmwaw::ParseException());
   bool ok = true;
   try {
     // create the asciiFile
@@ -453,7 +455,8 @@ bool MsWrd1Parser::createZones()
   for (int z = 5; z >= 0; z--) {
     if (m_state->m_fileZonesLimit[z] == m_state->m_fileZonesLimit[z+1])
       continue;
-    if (!input->checkPosition(m_state->m_fileZonesLimit[z+1]*0x80)) {
+    if (!input->checkPosition(m_state->m_fileZonesLimit[z+1]*0x80) ||
+        m_state->m_fileZonesLimit[z] > m_state->m_fileZonesLimit[z+1]) {
       f.str("");
       f << "Entries(Zone" << z << "):###";
       MWAW_DEBUG_MSG(("MsWrd1Parser::createZones: zone %d is too long\n",z));
@@ -461,7 +464,7 @@ bool MsWrd1Parser::createZones()
       ascii().addNote(f.str().c_str());
       break;
     }
-    Vec2i limit(m_state->m_fileZonesLimit[z],m_state->m_fileZonesLimit[z+1]);
+    MWAWVec2i limit(m_state->m_fileZonesLimit[z],m_state->m_fileZonesLimit[z+1]);
     bool done = false;
     switch (z) {
     case 0:
@@ -513,7 +516,7 @@ bool MsWrd1Parser::prepareTextZones()
   }
   if (endMain < 0x80) {
     MWAW_DEBUG_MSG(("MsWrd1Parser::sendText: oops problem computing the limit of the main section"));
-    m_state->m_textZonesList.push_back(Vec2l(0x80, m_state->m_eot));
+    m_state->m_textZonesList.push_back(MWAWVec2l(0x80, m_state->m_eot));
     m_state->m_mainTextZonesList.push_back(0);
     return false;
   }
@@ -522,7 +525,7 @@ bool MsWrd1Parser::prepareTextZones()
   long pos = 0x80, prevMainPos=pos;
   int actPage = 1;
   int actType = 0;
-  Vec2i headerId(-1,-1), footerId(-1,-1);
+  MWAWVec2i headerId(-1,-1), footerId(-1,-1);
   int firstHeaderId=-1, firstFooterId=-1;
   while (pos < endMain) {
     int newType = 0;
@@ -556,7 +559,7 @@ bool MsWrd1Parser::prepareTextZones()
     }
 
     int id = int(m_state->m_textZonesList.size());
-    m_state->m_textZonesList.push_back(Vec2l(prevMainPos, pos));
+    m_state->m_textZonesList.push_back(MWAWVec2l(prevMainPos, pos));
     prevMainPos=pos;
     if (actType==0) {
       m_state->m_mainTextZonesList.push_back(id);
@@ -632,7 +635,7 @@ bool MsWrd1Parser::readFont(long fPos, MsWrd1ParserInternal::Font &font)
     case 0:
       break;
     case 3:
-      flags |= MWAWFont::allCapsBit;
+      flags |= MWAWFont::uppercaseBit;
       break;
     default:
       f << "#capBits=" << int((val&0xc)>>2) << ",";
@@ -795,7 +798,7 @@ bool MsWrd1Parser::readParagraph(long fPos, MsWrd1ParserInternal::Paragraph &par
 }
 
 /* read the page break separation */
-bool MsWrd1Parser::readPageBreak(Vec2i limits)
+bool MsWrd1Parser::readPageBreak(MWAWVec2i limits)
 {
   MWAWInputStreamPtr input = getInput();
   if (limits[1] <= limits[0] || !input->checkPosition(limits[1]*0x80)) {
@@ -840,7 +843,7 @@ bool MsWrd1Parser::readPageBreak(Vec2i limits)
 }
 
 /* read the footnote zone */
-bool MsWrd1Parser::readFootnoteCorrespondance(Vec2i limits)
+bool MsWrd1Parser::readFootnoteCorrespondance(MWAWVec2i limits)
 {
   MWAWInputStreamPtr input = getInput();
   if (limits[1] <= limits[0] || !input->checkPosition(limits[1]*0x80)) {
@@ -886,7 +889,7 @@ bool MsWrd1Parser::readFootnoteCorrespondance(Vec2i limits)
   m_state->m_footnotesList.resize(footnoteMap.size(),0);
   std::map<long, int>::iterator fIt=footnoteMap.begin();
   for (fIt=footnoteMap.begin(); fIt!=footnoteMap.end();) {
-    Vec2l fPos;
+    MWAWVec2l fPos;
     fPos[0] = fIt->first;
     int id = fIt++->second;
     fPos[1] = fIt==footnoteMap.end() ? m_state->m_eot : fIt->first;
@@ -902,7 +905,7 @@ bool MsWrd1Parser::readFootnoteCorrespondance(Vec2i limits)
 }
 
 /* read the zone4: a list of main zone ( headers, footers ) ? */
-bool MsWrd1Parser::readZones(Vec2i limits)
+bool MsWrd1Parser::readZones(MWAWVec2i limits)
 {
   MWAWInputStreamPtr input = getInput();
   if (limits[1] <= limits[0] || !input->checkPosition(limits[1]*0x80)) {
@@ -949,7 +952,7 @@ bool MsWrd1Parser::readZones(Vec2i limits)
 }
 
 /* read the document information */
-bool MsWrd1Parser::readDocInfo(Vec2i limits)
+bool MsWrd1Parser::readDocInfo(MWAWVec2i limits)
 {
   MWAWInputStreamPtr input = getInput();
   if (limits[1] != limits[0]+1 || !input->checkPosition(limits[1]*0x80)) {
@@ -1088,7 +1091,7 @@ bool MsWrd1Parser::readDocInfo(Vec2i limits)
 }
 
 // read a plc zone (char or paragraph properties )
-bool MsWrd1Parser::readPLC(Vec2i limits, int wh)
+bool MsWrd1Parser::readPLC(MWAWVec2i limits, int wh)
 {
   MWAWInputStreamPtr input = getInput();
   if (limits[1] <= limits[0] || !input->checkPosition(limits[1]*0x80)) {
@@ -1297,7 +1300,7 @@ bool MsWrd1Parser::sendText(MWAWEntry const &textEntry, bool isMain)
     f << (char) c;
     switch (c) {
     case 1:
-      getTextListener()->insertUnicodeString("(picture)");
+      getTextListener()->insertUnicodeString(librevenge::RVNGString("(picture)"));
       break;
     case 5: // footnote mark
     case 0xc: // end of file
