@@ -139,7 +139,7 @@ struct Picture {
   }
 
   //! the dimension
-  Box2i m_dim;
+  MWAWBox2i m_dim;
   //! the list of picture
   std::vector<Zone> m_picturesList;
   //! an unknown flag
@@ -163,7 +163,7 @@ struct Picture {
     //! the position in file
     MWAWEntry m_pos;
     //! the dimension
-    Box2i m_dim;
+    MWAWBox2i m_dim;
     //! three unknown flags
     int m_flags[3];
   };
@@ -246,18 +246,22 @@ protected:
 void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType type)
 {
   if (!listener.get()) {
-    MWAW_DEBUG_MSG(("SubDocument::parse: no listener\n"));
+    MWAW_DEBUG_MSG(("MsWrdParserInternal::SubDocument::parse: no listener\n"));
+    return;
+  }
+  MsWrdParser *parser=dynamic_cast<MsWrdParser *>(m_parser);
+  if (!parser) {
+    MWAW_DEBUG_MSG(("MsWrdParserInternal::SubDocument::parse: no parser\n"));
     return;
   }
-  assert(m_parser);
 
   long pos = m_input->tell();
   if (m_type == libmwaw::DOC_NONE && m_pictCPos >= 0 && m_pictFPos > 0)
-    static_cast<MsWrdParser *>(m_parser)->sendPicture(m_pictFPos, m_pictCPos, MWAWPosition::Frame);
+    parser->sendPicture(m_pictFPos, m_pictCPos, MWAWPosition::Frame);
   else if (m_type == libmwaw::DOC_HEADER_FOOTER)
-    static_cast<MsWrdParser *>(m_parser)->send(m_zone);
+    parser->send(m_zone);
   else
-    static_cast<MsWrdParser *>(m_parser)->send(m_id, type);
+    parser->send(m_id, type);
   m_input->seek(pos, librevenge::RVNG_SEEK_SET);
 }
 
@@ -401,9 +405,7 @@ void MsWrdParser::send(int id, libmwaw::SubDocumentType type)
 ////////////////////////////////////////////////////////////
 void MsWrdParser::parse(librevenge::RVNGTextInterface *docInterface)
 {
-  assert(getInput().get() != 0);
-
-  if (!checkHeader(0L))  throw(libmwaw::ParseException());
+  if (!getInput().get() || !checkHeader(0L))  throw(libmwaw::ParseException());
   bool ok = true;
   try {
     // create the asciiFile
@@ -1727,7 +1729,7 @@ bool MsWrdParser::readPicture(MsWrdEntry &entry)
   int dim[4];
   for (int i = 0; i < 4; i++)
     dim[i] = (int) input->readLong(2);
-  pict.m_dim=Box2i(Vec2i(dim[1],dim[0]), Vec2i(dim[3],dim[2]));
+  pict.m_dim=MWAWBox2i(MWAWVec2i(dim[1],dim[0]), MWAWVec2i(dim[3],dim[2]));
   f << pict;
   ascii().addPos(pos);
   ascii().addNote(f.str().c_str());
@@ -1749,7 +1751,7 @@ bool MsWrdParser::readPicture(MsWrdEntry &entry)
       zone.m_flags[i] = (int) input->readULong((i==2) ? 2 : 1);
     for (int i = 0; i < 4; i++)
       dim[i] = (int) input->readLong(2);
-    zone.m_dim=Box2i(Vec2i(dim[1],dim[0]), Vec2i(dim[3],dim[2]));
+    zone.m_dim=MWAWBox2i(MWAWVec2i(dim[1],dim[0]), MWAWVec2i(dim[3],dim[2]));
     zone.m_pos.setBegin(pos+16);
     zone.m_pos.setLength(sz-16);
     f << zone;
@@ -1787,7 +1789,7 @@ void MsWrdParser::sendPicture(long fPos, int cPos, MWAWPosition::AnchorTo anchor
     return;
   }
   if (m_state->m_picturesMap.find(fPos)==m_state->m_picturesMap.end()) {
-    MWAW_DEBUG_MSG(("MsWrdParser::sendPicture: can not find picture for pos %lx\n", fPos));
+    MWAW_DEBUG_MSG(("MsWrdParser::sendPicture: can not find picture for pos %lx\n", (long unsigned int) fPos));
     return;
   }
   MsWrdParserInternal::Picture const &pict= m_state->m_picturesMap.find(fPos)->second;
@@ -1803,7 +1805,7 @@ void MsWrdParser::sendPicture(long fPos, int cPos, MWAWPosition::AnchorTo anchor
     getTextListener()->insertTextBox(pictPos, subdoc);
     return;
   }
-  MWAWPosition basicPos(Vec2f(0.,0.), Vec2f(100.,100.), librevenge::RVNG_POINT);
+  MWAWPosition basicPos(MWAWVec2f(0.,0.), MWAWVec2f(100.,100.), librevenge::RVNG_POINT);
   if (anchor != MWAWPosition::Page && anchor != MWAWPosition::Frame) {
     basicPos.setRelativePosition(anchor, MWAWPosition::XLeft, MWAWPosition::YCenter);
     basicPos.m_wrapping =  MWAWPosition::WBackground;
@@ -1812,14 +1814,12 @@ void MsWrdParser::sendPicture(long fPos, int cPos, MWAWPosition::AnchorTo anchor
     basicPos.setRelativePosition(anchor);
 
   long actPos = input->tell();
-  std::string pictType;
-  librevenge::RVNGBinaryData data;
-  Box2f naturalBox;
+  MWAWBox2f naturalBox;
   for (size_t p = 0; p < pict.m_picturesList.size(); p++) {
     MsWrdParserInternal::Picture::Zone const &zone=pict.m_picturesList[p];
     if (!zone.m_pos.valid()) continue;
     MWAWPosition pos(basicPos);
-    pos.setOrigin(pos.origin()+(Vec2f)zone.m_dim.min());
+    pos.setOrigin(pos.origin()+(MWAWVec2f)zone.m_dim.min());
     pos.setSize(zone.m_dim.size());
 
     input->seek(zone.m_pos.begin(), librevenge::RVNG_SEEK_SET);
@@ -1832,9 +1832,9 @@ void MsWrdParser::sendPicture(long fPos, int cPos, MWAWPosition::AnchorTo anchor
     input->seek(zone.m_pos.begin(), librevenge::RVNG_SEEK_SET);
     shared_ptr<MWAWPict> thePict(MWAWPictData::get(input, (int)zone.m_pos.length()));
     if (!thePict) continue;
-    thePict->getBinary(data,pictType);
-    if (data.size())
-      getTextListener()->insertPicture(pos, data, pictType);
+    MWAWEmbeddedObject picture;
+    if (thePict->getBinary(picture))
+      getTextListener()->insertPicture(pos, picture);
   }
   input->seek(actPos, librevenge::RVNG_SEEK_SET);
 }
@@ -1858,20 +1858,20 @@ bool MsWrdParser::readPrintInfo(MsWrdEntry &entry)
   if (!info.read(input)) return false;
   f << "PrintInfo:"<< info;
 
-  Vec2i paperSize = info.paper().size();
-  Vec2i pageSize = info.page().size();
+  MWAWVec2i paperSize = info.paper().size();
+  MWAWVec2i pageSize = info.page().size();
   if (pageSize.x() <= 0 || pageSize.y() <= 0 ||
       paperSize.x() <= 0 || paperSize.y() <= 0) return false;
 
   // define margin from print info
-  Vec2i lTopMargin= -1 * info.paper().pos(0);
-  Vec2i rBotMargin=info.paper().size() - info.page().size();
+  MWAWVec2i lTopMargin= -1 * info.paper().pos(0);
+  MWAWVec2i rBotMargin=info.paper().size() - info.page().size();
 
   // move margin left | top
   int decalX = lTopMargin.x() > 14 ? lTopMargin.x()-14 : 0;
   int decalY = lTopMargin.y() > 14 ? lTopMargin.y()-14 : 0;
-  lTopMargin -= Vec2i(decalX, decalY);
-  rBotMargin += Vec2i(decalX, decalY);
+  lTopMargin -= MWAWVec2i(decalX, decalY);
+  rBotMargin += MWAWVec2i(decalX, decalY);
 
   int leftMargin = lTopMargin.x();
   int topMargin = lTopMargin.y();
