@@ -110,13 +110,17 @@ protected:
 void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType /*type*/)
 {
   if (!listener.get()) {
-    MWAW_DEBUG_MSG(("SubDocument::parse: no listener\n"));
+    MWAW_DEBUG_MSG(("WriteNowParserInternal::SubDocument::parse: no listener\n"));
+    return;
+  }
+  WriteNowParser *parser=dynamic_cast<WriteNowParser *>(m_parser);
+  if (!parser) {
+    MWAW_DEBUG_MSG(("WriteNowParserInternal::SubDocument::parse: no parser\n"));
     return;
   }
-  assert(m_parser);
 
   long pos = m_input->tell();
-  static_cast<WriteNowParser *>(m_parser)->send(m_pos);
+  parser->send(m_pos);
   m_input->seek(pos, librevenge::RVNG_SEEK_SET);
 }
 
@@ -205,7 +209,7 @@ void WriteNowParser::send(WriteNowEntry const &entry)
   m_textParser->send(entry);
 }
 
-bool WriteNowParser::sendGraphic(int gId, Box2i const &bdbox)
+bool WriteNowParser::sendGraphic(int gId, MWAWBox2i const &bdbox)
 {
   if (gId < 8 || (gId-8) >= int(m_state->m_picturesList.size())) {
     MWAW_DEBUG_MSG(("WriteNowParser::sendGraphic: called with bad id=%d\n", gId));
@@ -226,9 +230,7 @@ bool WriteNowParser::sendGraphic(int gId, Box2i const &bdbox)
 ////////////////////////////////////////////////////////////
 void WriteNowParser::parse(librevenge::RVNGTextInterface *docInterface)
 {
-  assert(getInput().get() != 0);
-
-  if (!checkHeader(0L))  throw(libmwaw::ParseException());
+  if (!getInput().get() || !checkHeader(0L))  throw(libmwaw::ParseException());
   bool ok = true;
   try {
     // create the asciiFile
@@ -247,7 +249,7 @@ void WriteNowParser::parse(librevenge::RVNGTextInterface *docInterface)
 
       // ok, we can now send what is not send.
       m_textParser->flushExtra();
-      Box2i emptyBdBox;
+      MWAWBox2i emptyBdBox;
       for (size_t i = 0; i < m_state->m_picturesList.size(); i++) {
         if (m_state->m_picturesList[i].isParsed() ||
             !m_state->m_picturesList[i].isZone()) continue;
@@ -627,7 +629,7 @@ bool WriteNowParser::parseGraphicZone(WriteNowEntry const &entry)
 ////////////////////////////////////////////////////////////
 // try to read a graphic zone
 ////////////////////////////////////////////////////////////
-bool WriteNowParser::sendPicture(WriteNowEntry const &entry, Box2i const &bdbox)
+bool WriteNowParser::sendPicture(WriteNowEntry const &entry, MWAWBox2i const &bdbox)
 {
   MWAWInputStreamPtr input = getInput();
 
@@ -674,19 +676,18 @@ bool WriteNowParser::sendPicture(WriteNowEntry const &entry, Box2i const &bdbox)
     }
     else {
       if (getTextListener()) {
-        librevenge::RVNGBinaryData data;
-        std::string pictType;
         MWAWPosition pictPos;
         if (bdbox.size().x() > 0 && bdbox.size().y() > 0) {
-          pictPos=MWAWPosition(Vec2f(0,0),bdbox.size(), librevenge::RVNG_POINT);
+          pictPos=MWAWPosition(MWAWVec2f(0,0),bdbox.size(), librevenge::RVNG_POINT);
           pictPos.setNaturalSize(pict->getBdBox().size());
         }
         else
-          pictPos=MWAWPosition(Vec2f(0,0),pict->getBdBox().size(), librevenge::RVNG_POINT);
+          pictPos=MWAWPosition(MWAWVec2f(0,0),pict->getBdBox().size(), librevenge::RVNG_POINT);
         pictPos.setRelativePosition(MWAWPosition::Char);
 
-        if (pict->getBinary(data,pictType))
-          getTextListener()->insertPicture(pictPos, data, pictType);
+        MWAWEmbeddedObject picture;
+        if (pict->getBinary(picture))
+          getTextListener()->insertPicture(pictPos, picture);
       }
 
 #ifdef DEBUG_WITH_FILES
@@ -859,20 +860,20 @@ bool WriteNowParser::readPrintInfo(WriteNowEntry const &entry)
   }
   f << info;
 
-  Vec2i paperSize = info.paper().size();
-  Vec2i pageSize = info.page().size();
+  MWAWVec2i paperSize = info.paper().size();
+  MWAWVec2i pageSize = info.page().size();
   if (pageSize.x() <= 0 || pageSize.y() <= 0 ||
       paperSize.x() <= 0 || paperSize.y() <= 0) return false;
 
   // define margin from print info
-  Vec2i lTopMargin= -1 * info.paper().pos(0);
-  Vec2i rBotMargin=info.paper().size() - info.page().size();
+  MWAWVec2i lTopMargin= -1 * info.paper().pos(0);
+  MWAWVec2i rBotMargin=info.paper().size() - info.page().size();
 
   // move margin left | top
   int decalX = lTopMargin.x() > 14 ? lTopMargin.x()-14 : 0;
   int decalY = lTopMargin.y() > 14 ? lTopMargin.y()-14 : 0;
-  lTopMargin -= Vec2i(decalX, decalY);
-  rBotMargin += Vec2i(decalX, decalY);
+  lTopMargin -= MWAWVec2i(decalX, decalY);
+  rBotMargin += MWAWVec2i(decalX, decalY);
 
   // decrease right | bottom
   int rightMarg = rBotMargin.x() -50;
